clear
close all
CenterFrequency=546.775e6;% poprawka na nosna
Ts=1/(5e6/512);% based on parameters of USRP used in the other file
which_case='measurements' % possible options: 'Rayleigh'/'measurements'/'LoS'

switch which_case
    case 'measurements'
        %%OPTION 1: READ SAMPLES FROM MEASUREMENT
        load('probki.mat')
        results=transpose(results);
        results=transpose(results(:));
        tt= time_meas';
        tt=tt(:)';
        tt=tt-tt(1);
    case 'Rayleigh'
        % OPTION 2
        % uncomment if want to generate a flat Rayleigh channel
        v_set=0.8;%m/s
        f_dopp=v_set/(3*10^8)*CenterFrequency;
        no_samples=200000;
        f_s=1/Ts;%or 1/Ts
        results=generate_rayleigh(no_samples,f_dopp,f_s);
        tt=[0:no_samples-1]*1/f_s;
    case 'LoS'
        %% OPTION 3: GENERATE SIMPLE LOS CHANNEL
        no_samples=200000;
        f_s=1/Ts;%or 1/Ts
        results=ones(1,no_samples)*exp(1j*rand(1)*2*pi);
        tt=[0:no_samples-1]*1/f_s;
end




%% BEGIN OF SIGNAL CORRECATION (REMOVE FREQUENCY OFFSET IN TWO PHASES) AND LARGE SCALE FADING (ADVANCED METHOD ASSUMING THE PATHLOSS IS NOT A CONSTANT BUT LINEARLY CHANGING OVER A MEASUREMENT)
% zaklada ze wszystkie probki sa ciagle
%estymacja i przesuniecie integer frequency offset
N=512;
fft_samples=fft(results(1:N));
[~,integer_f_off]=max(abs(fft_samples));
integer_f_off=mod(integer_f_off-1+N/2,N)-N/2% w zakresie od -N/2 do N/2-1
results=results.*exp(-1j*2*pi*integer_f_off/N*[1:length(results)]);




%a moze normalizowac, zeby srednia moc po
%pathloss=32*log10(d0+v*(t-t0))+A= (V moze byc ujemne- jako wsp. d duzo wieksze.
%h= 10^(pathloss/20+probka_shadowing)*h_ray
%h=10^(1.6*log10(d0+v*t)+B)*h_ray
%h=C*(d0+v*(t-t0))^1.6*h_ray= D*(1+v*(t-t0)/d0)^1.6*h_ray
% binomial series- przyblizenie taylorem (1+x)^alpha=1+alfa*x
%h=(D+E*(t-t0))*h_ray
%dobrac D oraz E zeby srednia moc E(abs(h_ray).^2)=1
%suma(|h_i|^2/(D+E*(t_i-t0))^2)= length(h)
% normalizacja
wspA=sqrt(mean(abs(results(1:round(length(results)/10))).^2));
wspB0=(sqrt(mean(abs(results(round(length(results)*9/10):end)).^2))-wspA)/tt(end);
% funkcja=@(wspB)sum(abs(results).^2./(wspA+wspB*tt).^2)-length(results);
% wspB=fzero(funkcja,wspB0);
funkcja=@(wsp)sum(abs(results).^2./(wsp(1)+wsp(2)*tt).^2)-length(results);
dd= funkcja([wspA wspB0])
[wsp_optimal,~,~,exitflag]=lsqnonlin(funkcja,[wspA wspB0]);
cc=funkcja(wsp_optimal)
if exitflag<=0
    warning('optymalizacja nie znalazla rozwiazania')
    wsp_optimal(1)=sqrt(mean(abs(results).^2));
    wsp_optimal(2)=0;
end

y=results./(wsp_optimal(1)+wsp_optimal(2)*tt);

freq_off=mean(angle(y(2:end)./y(1:end-1)),'omitnan');% estymacja offsetu czestotliwosci
y_kor=y.*exp(-1j*freq_off*[0:length(y)-1]);
autokor_re=xcorr(real(y_kor),'normalized');
autokor_im=xcorr(imag(y_kor),'normalized');


% dopasoawnie predkosci w funkcji bessla
v_test=[0.1:0.1:4];%m/s

bessel_test=besselj(0,2*pi*(v_test'/3e8*CenterFrequency)*[-(length(y_kor)-1):(length(y_kor)-1)]*Ts);
[~,ktora_real]=min(sum((bessel_test-autokor_re).^2,2));
[~,ktora_imag]=min(sum((bessel_test-autokor_im).^2,2));

figure
subplot(3,2,1)
plot([-(length(y)-1):(length(y)-1)]*Ts,autokor_re,'b','DisplayName','Re(x)')
hold on
plot([-(length(y)-1):(length(y)-1)]*Ts,bessel_test(ktora_real,:),'b--','DisplayName',['Re(x)- theory, v=' num2str(v_test(ktora_real)) 'm/s'])
plot([-(length(y)-1):(length(y)-1)]*Ts,autokor_im,'r','DisplayName','Im(x)')
plot([-(length(y)-1):(length(y)-1)]*Ts,bessel_test(ktora_imag,:),'r--','DisplayName',['Im(x)- theory, v=' num2str(v_test(ktora_imag)) 'm/s'])

grid
xlabel('czas (s)')
ylabel('Korelacja')
xlim([-2 2])
legend
subplot(3,2,2)
[y_hist,x_hist]=histcounts(real(y_kor),50,'Normalization','pdf') ;
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_hist,'b','DisplayName','Re(x)-measurement')
y_teor=normpdf(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),0,1/sqrt(2));
hold on
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_teor,'b--','DisplayName','Re(x)- theory')

[y_hist,x_hist]=histcounts(imag(y_kor),50,'Normalization','pdf') ;
grid
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_hist,'r','DisplayName','Im(x)-measurement')
y_teor=normpdf(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),0,1/sqrt(2));
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_teor,'r--','DisplayName','Im(x)-theory')
xlabel('sample')
ylabel('pdf')
legend
subplot(3,2,3)
[y_hist,x_hist]=histcounts(abs(y_kor),50,'Normalization','pdf') ;
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_hist,'b','DisplayName','measurement')
hold on
teor_y=raylpdf(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),1/sqrt(2));
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),teor_y,'b--','DisplayName','theory')
grid
legend
xlabel('amplitide')
ylabel('pdf')

subplot(3,2,4)
[y_hist,x_hist]=histcounts(angle(y_kor),50,'Normalization','pdf') ;
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_hist,'b','DisplayName','measurement')
hold
y_teor=unifpdf(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),-pi,pi);
plot(0.5*x_hist(2:end)+0.5*x_hist(1:end-1),y_teor,'b--','DisplayName','theory')
grid
xlabel('angle')
ylabel('pdf')
legend
wsp_korrelacji_Re_Im=corrcoef(real(y_kor)',imag(y_kor)')

subplot(3,2,[5 6])
plot(tt(1:50:end),abs(y_kor(1:50:end)))
grid
xlabel('time (s)')
ylabel('amplitude')
